--[[
Identifies the action information.  This includes the visual behavior of the 
associated action button (hidden, low alpha, blinking, etc.) and the action to 
execute when the button is clicked.  This class sets the action attributes on 
the ActionButton, and tells the ActionButton how to draw itself based on the 
visual effect settings and current unit state.  This class also controls 
whether the ActionButton is in use or not.  If not in use, the ActionButton is 
told to hide itself.

Each Action has one or more ActionButton instances associated with it. 
Each ActionButton instance is associated with a single Action instance.
here is one ActionButton for each action of the ActionSet associated with each 
UnitGroupHeader.
--]]
uab.classes.Action = uab.mixins.aceOO.Class()

function uab.classes.Action.ItemUsable(button, usable)
    if usable then
        getglobal(button:Frame():GetName().."Icon"):SetVertexColor(1.0, 1.0, 1.0, 1.0)
    else
        getglobal(button:Frame():GetName().."Icon"):SetVertexColor(0.25, 0.25, 0.25, 1.0)
    end
end

function uab.classes.Action.AnyChosenEffect(settings)
    if settings then
		if settings.highlight or settings.lowalpha or
			settings.noalpha or settings.blink or
			settings.notusable then
            return true
        end
    end
end
        
function uab.classes.Action.prototype:init(number, data, effects)
	uab.classes.Action.super.prototype.init(self)
	
    self.enabled = true
	self.number = number
	self.data = data
	self.effects = effects
	self.buttons = {}
end

function uab.classes.Action.prototype:Disable(force)
	for button, _ in pairs(self.buttons) do
		button:Disable()
	end
    
    self.enabled = nil
    self.forcedisable = force or self.forcedisable
end

function uab.classes.Action.prototype:Enable(force)
    if force or not self.forcedisable then
        self.enabled = true
        self.forcedisable = nil
    	for button, _ in pairs(self.buttons) do
    		button:Enable()
    	end
    end
end

function uab.classes.Action.prototype:Enabled()
    return self.enabled
end

function uab.classes.Action.prototype:SetAll(data, effects)
	self.data = data
	self.effects = effects
end

function uab.classes.Action.prototype:SetData(data)
	self:ClearButtonAttributes()
	self.data = data
	self:SetButtonAttributes()
end

function uab.classes.Action.prototype:GetData()
	return self.data
end

function uab.classes.Action.prototype:SetEffects(effects)
	self.effects = effects
end

function uab.classes.Action.prototype:ShowButton(button)
    self:SetButtonIcon(button)
    self:SetButtonCount(button)
    
    self:UpdateVisualEffects(button)
end

function uab.classes.Action.prototype:AddButton(button)
    self.buttons[button] = true
	button:SetAttributes(self.data:GetAttributes())
    self:SetButtonIcon(button)
    self:SetButtonCount(button)
end

function uab.classes.Action.prototype:RemoveButton(button)
	self.buttons[button] = nil
	button:ClearAttributes(self.data:GetAttributes())
    --getglobal(button:Frame():GetName().."Icon"):SetTexture("Interface\\AddOns\\UnitActionBars\\EmptySlot")
end

function uab.classes.Action.prototype:ShowTooltip(frame)
	if uab.db.profile.tooltips then
        self.data:ShowTooltip(frame)
    end
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.Action.prototype:ClearButtonAttributes()
	if self.data then
		local attribs = self.data:GetAttributes()
		for button, _ in pairs(self.buttons) do
			button:ClearAttributes(attribs)
		end
	end
end

function uab.classes.Action.prototype:SetButtonAttributes()
	if self.data then
		local attribs = self.data:GetAttributes()
		for button, _ in pairs(self.buttons) do
			button:SetAttributes(attribs)
		end
	end
end

function uab.classes.Action.prototype:SetButtonIcon(button)
    local icon = self.data:GetIcon()
    
    if icon then
        local iconFrame = getglobal(button:Frame():GetName().."Icon")
        if iconFrame then
            local curIcon = iconFrame:GetTexture()
            if curIcon ~= icon then
                iconFrame:SetTexture(icon)
            end
        end
    end
end

function uab.classes.Action.prototype:SetButtonCount(button)
    btncount = getglobal(button:Frame():GetName().."ItemCount")
    
    if self.data:GetCount() then
        btncount:Show()
        btncount:SetText(self.data:GetCount())
    else
        btncount:Hide()
    end
end

function uab.classes.Action.prototype:ApplyEffect(button, effect)
    if self.sample then
        button:BorderColor(1.0, 1.0, 1.0, 0.0)
        button:SetAlpha(1.0)
        button:EndFlash()
    else
	    if not effect.highlight and not effect.lowalpha and not effect.noalpha and not effect.blink and not effect.notusable then
	        if self.effects.default then
	            effect = self:CheckPriority(effect, "Default")
	        end
	    end

	    local usable

        usable = self:CheckItemIsUsable(button)

        if usable then
            usable = not effect.notusable
        end
        
        uab.classes.Action.ItemUsable(button, usable)
        
	    if effect.highlight then
	    	local color = effect.highlight.color

	        button:BorderColor(color.r, color.g, color.b, color.a)
	    else
	        button:BorderColor(1.0, 1.0, 1.0, 0.0)
	    end

        if effect.blink then
            button:StartFlash()
	    else
	        button:EndFlash()
        end

	    if effect.noalpha then
	        button:SetAlpha(0.0)
	    elseif effect.lowalpha then
	        button:SetAlpha(uab.db.profile.alpha)
	    else
            button:SetAlpha(1.0)
	    end
	end
end

function uab.classes.Action.prototype:CheckPriority(effect, setting)
    local newPriority = uabUnitStatus:StatusPriority(setting)
    local highlight = uabUnitStatus:StatusColor(setting)

    if self.effects[setting].highlight then
        if not effect.highlight or newPriority > effect.highlight.priority then
            effect.highlight = { color = highlight }
            effect.highlight.priority = newPriority
        end
    end
    
    if self.effects[setting].lowalpha then
        effect.lowalpha = true
    end
    
    if self.effects[setting].noalpha then
        effect.noalpha = true
    end
    
    if self.effects[setting].blink then
        effect.blink = true
    end
    
    if self.effects[setting].notusable then
        effect.notusable = true
    end
    
    return effect
end

function uab.classes.Action.prototype:CheckItemIsUsable(button)
	if self.data and uab.db.profile.checkusable then
        return self.data:IsUsable()
    else
        return true
    end
end

function uab.classes.Action.prototype:GetEffectInfo(button, unit, effect, setting)
	if uabUnitStatus:IsStatusGained(setting, unit, self.data) then
		effect = self:CheckPriority(effect, setting)
	end
    
    return effect
end

function uab.classes.Action.prototype:UpdateVisualEffects(button)
    local effect = {}

	if not self.sample then
        local unit = SecureButton_GetModifiedUnit(button:Frame())

	    if unit and self.effects then
			for cureffect, settings in pairs(self.effects) do
				--[[
				No need to check for an effect on the unit if having the effect
				makes no change to any of the button attributes.
				--]]
			    if cureffect ~= "Default" and uab.classes.Action.AnyChosenEffect(settings) then
			        effect = self:GetEffectInfo(button, unit, effect, cureffect)
			    end
			end
		end
	end

    self:ApplyEffect(button, effect)
end

