--[[
This class represents a specific set of units of the same type (player, party,
pet, partypet, raid, target, etc.).

A UnitGroup has a single set of Action instances.
A UnitGroup has one or more UnitGroupHeader instances.  How many it has depends
on WoW, unit frame add-ons, and group configuration.  WoW 2.01 has a bug with
frame-unit associations that force a UnitGroupHeader instance for each unit
frame (WoW 2.02 should fix this)..
--]]
uab.classes.UnitGroup = uab.mixins.aceOO.Class()

function uab.classes.UnitGroup.CreateAction(number, data)
	local actiondata
	local action

	if uab.classes[data.classname] then
		actiondata = uab.classes[data.classname]:new(data.data)
		action = uab.classes.Action:new(number, actiondata, data.effect)
	end

	return action
end

function uab.classes.UnitGroup.prototype:init(activator, name, numunits, defn)
	uab.classes.UnitGroup.super.prototype.init(self)

	self.name = name
	self.activator = activator
	self.qualifiedname = self.activator.name .. self.name
	self.numunits = numunits
	self.actions = {}
	self.numheaders = 1
	self.registered = {}
	self.defn = defn

    if self.defn.actions then
    	for idx = 1, 24, 1 do
            local action = self.defn.actions[idx]
    		if action then
                if action.classname then
    		        self.actions[idx] = uab.classes.UnitGroup.CreateAction(idx, action)
                else
                    self.defn.actions[idx] = nil
                end
    		end
    	end
    else
        self.defn.actions = {}
    end

	self.header = uab.classes.UnitGroupHeader:new(self, self.numunits, self.actions)

	if self.defn.enabled then
	    self:Enable()
	else
	    self:Disable()
	end
	
    --[[
    Make sure that the buttons are set to the correct scale
    --]]
    self:ScaleChanged()
    self:SwitchToSingleAction()
end

function uab.classes.UnitGroup.prototype:NewControl()
	local control = self.header:NewControl()

    if self.singleaction then
    	self.header:SwitchToSingleAction(self:FindOnlyAction(), control)
    else
    	self.header:SwitchToButtons(control)
    end

    return control
end

function uab.classes.UnitGroup.prototype:CopyTo(target)
    --[[
    Copy all the actions to the target group.  This means duplicating the action, then
    assigning the action to the same spot in the target group.  It also means that if
    we do not have an action for a specific position, we must clear the action for that
    position in the target group as well.
    --]]

    for idx = 1, 24, 1 do
        local data = nil
        
        if self.defn.actions[idx] then
            data = uab.CloneTable(self.defn.actions[idx])
        end

        target:SetAction(idx, data)
    end
end

function uab.classes.UnitGroup.prototype:IAmDead()
	self.header:IAmDead()
end

function uab.classes.UnitGroup.prototype:TimerEvent(elapsed)
	self.header:TimerEvent(elapsed)
end

function uab.classes.UnitGroup.prototype:SetAction(number, data)
    if data and data.classname then
    	self.defn.actions[number] = data
    	
    	if self.actions[number] then
    		local actiondata = uab.classes[data.classname]:new(data.data)
    	    
            self.actions[number]:SetAll(actiondata, data.effect)
    	    self.actions[number]:Enable(true)
    	else
    	    self.actions[number] = uab.classes.UnitGroup.CreateAction(number, data)
    	end
    
    	self.header:SetAction(number, self.actions[number])

        --[[
        Make sure that the buttons are properly arranged according to the
        scale and current layout
        --]]
        
        self:ScaleChanged()    
        self:SetLayout(self.layout)
    else
        self:ClearAction(number)
    end
    
    self:SwitchToSingleAction()
end

function uab.classes.UnitGroup.prototype:ClearAction(number)
	self.defn.actions[number] = nil
	if self.actions[number] then
	    self.actions[number]:Disable(true)
 	end
    
    self:SwitchToSingleAction()
end

function uab.classes.UnitGroup.prototype:FindOnlyAction()
    local onlyaction

    if self.activator.name == "cleanse" then
        uab.pdebug("SwitchToSingleAction(UnitGroup): "..self.qualifiedname)
    end

    for idx = 1, 24, 1 do
        if self.defn.actions[idx] then
            if onlyaction then
                return nil
            end

            onlyaction = self.actions[idx]
        end
    end
    
    return onlyaction
end

function uab.classes.UnitGroup.prototype:SwitchToSingleAction()
    if self.defn.actions and uab.db.profile.directcast then
        local onlyaction = self:FindOnlyAction()

        if onlyaction then        
            --[[
            If we get here, we only have a single action assigned to this group.
            Change that single action to execute when the control is clicked instead
            of coming up as a button.
            --]]
    
            self.singleaction = true    
        	self.header:SwitchToSingleAction(onlyaction)
        else
            self:SwitchToButtons()
        end
    else
        self:SwitchToButtons()
    end
end

function uab.classes.UnitGroup.prototype:SwitchToButtons()
    self:CheckCancelVisible()
    
    if self.singleaction then
        self.singleaction = nil
    	self.header:SwitchToButtons()
    end
end

function uab.classes.UnitGroup.prototype:SetLayout(layout)
	self.layout = layout
	self.header:LayoutChanged()
end

function uab.classes.UnitGroup.prototype:ResetButtons()
	self.header:ResetButtons()
end

function uab.classes.UnitGroup.prototype:UsingDefaultScale()
	local scale = uab.db.profile.scale[self.name]
    
    return scale.usedefault
end

function uab.classes.UnitGroup.prototype:GetScale()
	local scale = uab.db.profile.scale[self.name]
    
    if scale.usedefault then
        return uab.db.profile.scale.default.value
    else
        return scale.value
    end
end

function uab.classes.UnitGroup.prototype:UsingDefaultPosition()
	local pos = uab.db.profile.position[self.name]
    
    return pos.usedefault
end

function uab.classes.UnitGroup.prototype:GetPosition()
	local pos = uab.db.profile.position[self.name]
    
    if pos.usedefault then
        return uab.db.profile.position.default
    else
        return pos
    end
end

function uab.classes.UnitGroup.prototype:SecureHeaderChanged()
	self.header:SecureHeaderChanged()
end

function uab.classes.UnitGroup.prototype:ScaleChanged()
	self.header:ScaleChanged()
end

function uab.classes.UnitGroup.prototype:PositionChanged()
	self.header:PositionChanged()
end

function uab.classes.UnitGroup.prototype:DirectCastChanged()
    self:SwitchToSingleAction()
end

function uab.classes.UnitGroup.prototype:StayOpenChanged()
	self.header:StayOpenChanged()
end

function uab.classes.UnitGroup.prototype:HighlightBorderChanged()
	self.header:HighlightBorderChanged()
end

function uab.classes.UnitGroup.prototype:IsStayOpen()
    local groupsetting = uab.db.profile.groupstayopen[self.name]
     
    if not groupsetting then
        return uab.db.profile.stayopen
    end

    return groupsetting.enabled    
end

function uab.classes.UnitGroup.prototype:FastChangeUnitChanged()
	self.header:FastChangeUnitChanged()
end

function uab.classes.UnitGroup.prototype:SetFrameGroupFrameEnabledStates()
	self.header:SetFrameGroupFrameEnabledStates()
end

function uab.classes.UnitGroup.prototype:RegisterFrame(frame)
	if not self.registered[frame] then
		--[[
		Need to associate the frame with some control button in the group.
		--]]
        local found
		
        if self.header:CanRegisterFrame(frame) then
			self.header:RegisterFrame(frame)
    	    self.registered[frame] = true
            found = true
		end
		
	    if not found then
            local unit = SecureButton_GetModifiedUnit(frame)
            
            if unit then
                uab.print("Unable to register frame : "..frame:GetName().." ("..unit..") in group : "..self.name)
            else
                uab.print("Unable to register frame : "..frame:GetName().." (no unit) in group : "..self.name)
            end
        end
    end
end

function uab.classes.UnitGroup.prototype:UnregisterFrame(frame)
	if self.registered[frame] then
		--[[
		Need to disassociate the frame with the control button that is currently associated with it.
		--]]
        if self.header:IsRegisteredFrame(frame) then
			self.header:UnregisterFrame(frame)
        	self.registered[frame] = nil
		end
    end
end

function uab.classes.UnitGroup.prototype:Enable()
    self:CheckCancelVisible()
	self.defn.enabled = true
	self.header:Enable()
end

function uab.classes.UnitGroup.prototype:Disable()
	self.defn.enabled = nil
	self.header:Disable()
end

function uab.classes.UnitGroup.prototype:CheckCancelVisible()
    local found
    
    for idx = 1, 24, 1 do
        if self.defn.actions[idx] then
            found = true
            break
        end
    end

    if found then
        if self.defn.enabled then
            self.header:Enable(true)
        end
    else
        self.header:Disable(true)
    end
end

function uab.classes.UnitGroup.prototype:Enabled()
	return self.defn.enabled
end