--[[
This class controls the arrangement of the ActionButtonNone and ActionButton
instances for a UnitGroupHeader.

There are one or more IActionSetLayout instances allocated for each
UnitGroupHeader.  How many depends on whether the layout is changed while
in-game.  The only layouts allocated are ones that were selected in-game.  Each
UnitGroupHeader keeps track of the active ActionSetLayout to use.
--]]
uab.classes.BarLayout = uab.mixins.aceOO.Class(IActionSetLayout)

function uab.classes.BarLayout.prototype:init(settings)
	uab.classes.BarLayout.super.prototype.init(self)
	
    self.config = uab.classes.BarLayoutConfig:new(self, settings)
end

function uab.classes.BarLayout.prototype:ArrangeButtons(cancelbtn, actionbtns)
	local settings = self.config.settings

	--[[
	First, reset all positioning information
	--]]
    cancelbtn:Frame():ClearAllPoints()
    self:FormatCancelBtn(cancelbtn)

	for idx = 1, 24, 1 do
	    if actionbtns[idx] then
	        actionbtns[idx]:Frame():ClearAllPoints()
            self:FormatActionBtn(actionbtns[idx])
		end
	end
	
	--[[
    Now, position the cancel button centered relative to the header (the
    parent of the cancel button)
    --]]
    cancelbtn:Frame():SetPoint("CENTER", 0, 0)

    local overflowModifier = { x = 0, y = 0 }
    local buttonRows = 24 / settings.growLimit

    if buttonRows * settings.growLimit < 24 then
        buttonRows = buttonRows + 1
    end

    if settings.overflowClockwise == "true" then
        if settings.growDirection == uab.constants.grow.up then
            overflowModifier.x = 1
            overflowModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.down then
            overflowModifier.x = -1
            overflowModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.right then
            overflowModifier.x = 0
            overflowModifier.y = -1
        else
            overflowModifier.x = 0
            overflowModifier.y = 1
        end
    else
        if settings.growDirection == uab.constants.grow.up then
            overflowModifier.x = -1
            overflowModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.down then
            overflowModifier.x = 1
            overflowModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.right then
            overflowModifier.x = 0
            overflowModifier.y = 1
        else
            overflowModifier.x = 0
            overflowModifier.y = -1
        end
    end

    local growModifier = { x = 0, y = 0 }
    local start = { x = 0, y = 0 }

    if settings.placement == uab.constants.placement.top then
        if settings.growDirection == uab.constants.grow.up then
            growModifier.x = 0
            growModifier.y = 1
        elseif settings.growDirection == uab.constants.grow.left then
            growModifier.x = -1
            growModifier.y = 0
        else    -- uab.constants.grow.right
            growModifier.x = 1
            growModifier.y = 0
        end

        start.x = 0
        start.y = 1
    elseif settings.placement == uab.constants.placement.right then
        if settings.growDirection == uab.constants.grow.right then
            growModifier.x = 1
            growModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.up then
            growModifier.x = 0
            growModifier.y = 1
        else    -- uab.constants.grow.down
            growModifier.x = 0
            growModifier.y = -1
        end

        start.x = 1
        start.y = 0
    elseif settings.placement == uab.constants.placement.bottom then
        if settings.growDirection == uab.constants.grow.down then
            growModifier.x = 0
            growModifier.y = -1
        elseif settings.growDirection == uab.constants.grow.right then
            growModifier.x = 1
            growModifier.y = 0
        else    -- uab.constants.grow.left
            growModifier.x = -1
            growModifier.y = 0
        end

        start.x = 0
        start.y = -1
    elseif settings.placement == uab.constants.placement.left then
        if settings.growDirection == uab.constants.grow.left then
            growModifier.x = -1
            growModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.up then
            growModifier.x = 0
            growModifier.y = 1
        else    -- uab.constants.grow.down
            growModifier.x = 0
            growModifier.y = -1
        end

        start.x = -1
        start.y = 0
    end

    local button = actionbtns[1]
    local positions = {}

    positions[1] = { x = start.x * (uab.constants.buttonWidth + 1), y = start.y * (uab.constants.buttonHeight + 1) }

    if button then
        button:Frame():SetPoint("CENTER", cancelbtn:Frame(), "CENTER", positions[1].x, positions[1].y)
    end

    for pos = 2, settings.growLimit, 1 do
        positions[pos] = { x = 0, y = 0 }

        positions[pos].x = (start.x + (growModifier.x * (pos - 1))) * (uab.constants.buttonWidth + 1)
        positions[pos].y = (start.y + (growModifier.y * (pos - 1))) * (uab.constants.buttonHeight + 1)

        button = actionbtns[pos]

        if button then
            button:Frame():SetPoint("CENTER", cancelbtn:Frame(), "CENTER", positions[pos].x, positions[pos].y)
        end
    end

    for row = 2, buttonRows, 1 do
        self:ArrangeRow(positions, row, overflowModifier, cancelbtn, actionbtns)
    end
end

function uab.classes.BarLayout.prototype:ArrangeRow(positions, row, placementMod, cancelbtn, actionbtns)
	local settings = self.config.settings
    local ofs = (row - 1) * settings.growLimit
    local relofs = ofs - settings.growLimit

    for pos = 1, settings.growLimit, 1 do
        local btnpos = pos + ofs
        local btn = actionbtns[btnpos]
        local relbtnpos = pos + relofs
        local relpos = positions[relbtnpos]

        positions[btnpos] = { x = 0, y = 0 }

        positions[btnpos].x = relpos.x + (placementMod.x * (uab.constants.buttonWidth + 1))
        positions[btnpos].y = relpos.y + (placementMod.y * (uab.constants.buttonHeight + 1))

        if btn then
            btn:Frame():SetPoint("CENTER", cancelbtn:Frame(), "CENTER", positions[btnpos].x, positions[btnpos].y)
        end
    end
end

function uab.classes.BarLayout.prototype:GetConfig()
	return self.config
end

function uab.classes.BarLayout.prototype:SetSettings(settings)
    self.config:SettingsChanged(settings)
end

function uab.classes.BarLayout.prototype:FormatCancelBtn(cancelbtn)
    -- Nothing to do here at this time.  Would be a good idea to resize
    -- the cancel button frame, hide the text frame for the button frame
end

function uab.classes.BarLayout.prototype:FormatActionBtn(btn)
    -- Nothing to do here at this time.  Would be a good idea to resize
    -- the cancel button frame, hide the text frame for the button frame
end

